/*
 * Copyright (c) 2005-2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.ui.editors.template;

import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

/**
 * A class for template content outline pages. This content outline page will be
 * presented to the user via the standard Content Outline View (the user decides
 * whether their workbench window contains this view) whenever that template
 * editor is active.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoTemplateOutlinePage extends ContentOutlinePage {

	/**
	 * Template editor.
	 */
	protected AcceleoTemplateEditor editor;

	/**
	 * Constructor.
	 * 
	 * @param editor
	 *            is the template editor
	 */
	public AcceleoTemplateOutlinePage(AcceleoTemplateEditor editor) {
		super();
		this.editor = editor;
	}

	/* (non-Javadoc) */
	public void createControl(Composite parent) {
		super.createControl(parent);
		TreeViewer viewer = getTreeViewer();
		viewer.setContentProvider(createContentProvider());
		viewer.setLabelProvider(createLabelProvider());
		if (editor.getGenerator() != null) {
			setInput(editor.getGenerator());
		}
	}

	/**
	 * Creates the content provider.
	 * 
	 * @return the content provider
	 */
	protected IContentProvider createContentProvider() {
		return new AcceleoTemplateContentProvider(editor);
	}

	/**
	 * Creates the label provider.
	 * 
	 * @return the label provider
	 */
	protected IBaseLabelProvider createLabelProvider() {
		return new AcceleoTemplateLabelProvider(editor);
	}

	/**
	 * Updates the input model.
	 * 
	 * @param model
	 *            is the new input model
	 */
	public void setInput(Object model) {
		if (getTreeViewer() != null)
			getTreeViewer().setInput(model);
	}

}
